#include "mainwindow.h"
#include <QFileDialog>
#include <QMessageBox>
#include <QContextMenuEvent>
#include <QMenu>
#include <QMenuBar>
#include <QFile>
#include <QApplication>
#include <QVBoxLayout>
#include <QTextStream>

MainWindow::MainWindow(QWidget *parent) : QMainWindow(parent){
    saveAct = new QAction(tr("Save"), this);
    saveAct->setShortcuts(QKeySequence::Save);
    connect(saveAct, &QAction::triggered, this, &MainWindow::save);

    newAct = new QAction(tr("New"), this);
    newAct->setShortcuts(QKeySequence::New);
    connect(newAct, &QAction::triggered, this, &MainWindow::clear);

    exitAct = new QAction(tr("Exit"), this);
    exitAct->setShortcuts(QKeySequence::Quit);
    connect(exitAct, &QAction::triggered, this, &QWidget::close);

    aboutAct = new QAction(tr("About"), this);
    connect(aboutAct, &QAction::triggered, this, &MainWindow::about);

    aboutQtAct = new QAction(tr("About Qt"), this);
    connect(aboutQtAct, &QAction::triggered, qApp, &QApplication::aboutQt);

    fileMenu = menuBar()->addMenu(tr("File"));
    fileMenu->addAction(saveAct);
    fileMenu->addAction(newAct);
    fileMenu->addSeparator();
    fileMenu->addAction(exitAct);
    helpMenu = menuBar()->addMenu(tr("Help"));
    helpMenu->addAction(aboutAct);
    helpMenu->addAction(aboutQtAct);
    setWindowTitle(tr("Food Calories Calculator"));
    QVBoxLayout *layout = new QVBoxLayout;

    QHBoxLayout *weightLayout = new QHBoxLayout;
    QLabel *weightLabel = new QLabel(tr("Weight (kg):"), this);
    weightLayout->addWidget(weightLabel);
    weightLineEdit = new QLineEdit(this);
    weightLineEdit->setValidator(new QDoubleValidator(this));
    weightLayout->addWidget(weightLineEdit);

    layout->addLayout(weightLayout);

    QHBoxLayout *chickenMeatLayout = new QHBoxLayout;
    QLabel *chickenMeatLabel = new QLabel(tr("Chicken Meat (kg):"), this);
    chickenMeatLayout->addWidget(chickenMeatLabel);
    chickenMeatLineEdit = new QLineEdit(this);
    chickenMeatLayout->addWidget(chickenMeatLineEdit);

    layout->addLayout(chickenMeatLayout);

    QHBoxLayout *milkLayout = new QHBoxLayout;
    QLabel *milkLabel = new QLabel(tr("Milk (kg):"), this);
    milkLayout->addWidget(milkLabel);
    milkLineEdit = new QLineEdit(this);
    milkLayout->addWidget(milkLineEdit);

    layout->addLayout(milkLayout);

    QHBoxLayout *chickenEggLayout = new QHBoxLayout;
    QLabel *chickenEggLabel = new QLabel(tr("Chicken Egg (pcs):"), this);
    chickenEggLayout->addWidget(chickenEggLabel);
    chickenEggLineEdit = new QLineEdit(this);
    chickenEggLayout->addWidget(chickenEggLineEdit);

    layout->addLayout(chickenEggLayout);

    submitButton = new QPushButton(tr("Calculate Calories"), this);
    connect(submitButton, &QPushButton::clicked, this, &MainWindow::handleSubmit);
    layout->addWidget(submitButton);

    textEdit = new QTextEdit(this);
    layout->addWidget(textEdit);

    QWidget *centralWidget = new QWidget(this);
    centralWidget->setLayout(layout);
    setCentralWidget(centralWidget);
}

void MainWindow::handleSubmit() {
    QString weightStr = weightLineEdit->text();
    QString chickenMeatStr = chickenMeatLineEdit->text();
    QString milkStr = milkLineEdit->text();
    QString chickenEggStr = chickenEggLineEdit->text();

    if (weightStr.isEmpty() || chickenMeatStr.isEmpty() || milkStr.isEmpty() || chickenEggStr.isEmpty()) {
        QMessageBox::warning(this, tr("Warning"), tr("Please enter all values."));
        return;
    }

    bool ok;
    double weight = weightStr.toDouble(&ok);
    if (!ok) {
        QMessageBox::warning(this, tr("Warning"), tr("Invalid weight format."));
        return;
    }

    double chickenMeat = chickenMeatStr.toDouble(&ok);
    if (!ok) {
        QMessageBox::warning(this, tr("Warning"), tr("Invalid chicken meat format."));
        return;
    }

    double milk = milkStr.toDouble(&ok);
    if (!ok) {
        QMessageBox::warning(this, tr("Warning"), tr("Invalid milk format."));
        return;
    }

    int chickenEgg = chickenEggStr.toInt(&ok);
    if (!ok) {
        QMessageBox::warning(this, tr("Warning"), tr("Invalid chicken egg format."));
        return;
    }

    // Обчислення кількості калорій та допустимої кількості їжі для даної ваги
    CChickenMeat chickenMeatFood;
    double chickenMeatCalories = chickenMeatFood.GetAmountForOnePersonPerDay(weight);

    CMilk milkFood;
    double milkCalories = milkFood.GetAmountForOnePersonPerDay(weight);

    CChickenEgg chickenEggFood;
    double chickenEggCalories = chickenEggFood.GetAmountForOnePersonPerDay(weight);

    // Виведення результатів у текстове поле
    textEdit->clear();
    textEdit->append(tr("Recommended daily food amount for a person weighing ") + QString::number(weight, 'f', 2) + tr(" kg:") +
                     "\nChicken Meat: " + QString::number(chickenMeatCalories, 'f', 2) + " kg" +
                     "\nMilk: " + QString::number(milkCalories, 'f', 2) + " kg" +
                     "\nChicken Eggs: " + QString::number(chickenEggCalories, 'f', 2) + " pcs");

    // Виведення кількості калорій для введеної ваги їжі
    textEdit->append(tr("\nCalories for the entered weight of food:") +
                     "\nChicken Meat: " + QString::number(chickenMeatFood.GetCaloriesForOneKg() * chickenMeat, 'f', 2) + " kcal" +
                     "\nMilk: " + QString::number(milkFood.GetCaloriesForOneKg() * milk, 'f', 2) + " kcal" +
                     "\nChicken Eggs: " + QString::number(chickenEggFood.GetCaloriesForOneKg() * chickenEgg, 'f', 2) + " kcal");
}

#ifndef QT_NO_CONTEXTMENU
void MainWindow::contextMenuEvent(QContextMenuEvent *event)
{
    QMenu menu(this);

    menu.addAction(saveAct);
    menu.addAction(newAct);
    menu.addAction(aboutAct);
    menu.exec(event->globalPos());
}
#endif // QT_NO_CONTEXTMENU

void MainWindow::save()
{
    QString fileName = QFileDialog::getSaveFileName(this, tr("Save File"), "", tr("Text Files (*.txt)"));
    if (fileName.isEmpty()) return;
    QFile file(fileName);
    if (!file.open(QIODevice::WriteOnly | QIODevice::Text)) {
        QMessageBox::warning(this, tr("Error"), tr("Could not open file for writing."));
        return;
    }

    QTextStream out(&file);
    out << textEdit->toPlainText();
    if (!textEdit->toPlainText().endsWith("\n")) out << '\n';

    file.close();
}

void MainWindow::about()
{
    QMessageBox::about(this, tr("About Food Calories Calculator"),
                       tr("Food calories calculator made by Lukavetskyi Bohdan"));
}

void MainWindow::clear()
{
    textEdit->clear();
}
